#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-04 $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''

Create or add to a database.  First attempt.  This could be made better/faster.
Assumes that all ais messages have been "normalized" - messages are one line only.

Only deals with sqlite right now.

@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: GPL v2
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector
import StringIO

import ais.binary    as binary
#import ais.aisstring as aisstring

import ais.ais_msg_1
import ais.ais_msg_2
import ais.ais_msg_3
import ais.ais_msg_5
#import ais.ais_msg_8
#import ais.ais_msg_21

import pysqlite2.dbapi2 as sqlite


def createTables(cx,verbose=False):
    '''
    param cx: database connection
    '''
    cu = cx.cursor()

    if verbose: print str(ais.ais_msg_1.sqlCreate(dbType='sqlite'))
    cu.execute(str(ais.ais_msg_1.sqlCreate(dbType='sqlite')))

    # Skip 2 and 3 since they are also position messages
    
    if verbose: print str(ais.ais_msg_5.sqlCreate(dbType='sqlite'))
    cu.execute(str(ais.ais_msg_5.sqlCreate(dbType='sqlite')))

    cx.commit()

def loadData(cx,datafile,verbose=False
	     , uscg=True):
    '''
    Try to read data from an open file object.  Not yet well tested.

    @param cx: database connection
    @param verbose: pring out more if true
    @param uscg: Process uscg tail information to get timestamp and receive station
    @rtype: None
    @return: Nothing

    @note: can not handle multiline AIS messages.  They must be normalized first.
    '''
    cu = cx.cursor()
    lineNum = 0

    counts = {1:0,2:0,3:0,5:0}

    for line in datafile:
	lineNum += 1
	if lineNum%1000==0:
	    print lineNum
	    cx.commit()
#	    if lineNum>3000:
#		print 'Early exit from load'
#		break

	if line[3:6] not in ('VDM|VDO'): continue # Not an AIS VHF message
	try:
	    msgNum = int(binary.ais6tobitvec(line.split(',')[5][0]))
	except:
	    print 'line would not decode',line
	    continue
	if verbose: print 'msgNum:',msgNum
	if msgNum not in (1,2,3,5): 
	    if verbose: print 'skipping',line
	    continue
	
	payload = bv = binary.ais6tobitvec(line.split(',')[5])

# FIX: need to take badding into account ... right before the *
	if msgNum in (1,2,3):
#	    if len(bv) != 168:
	    if len(bv) < 168:
		print 'ERROR: skipping bad position message, line:',lineNum
		print '  ',line,
		print '   Got length',len(bv), 'expected', 168
		continue
	elif msgNum == 5:
#	    if len(bv) != 424:
	    if len(bv) < 424:
		print 'ERROR: skipping bad shipdata message, line:',lineNum
		print '  ',line,
		print '   Got length',len(bv), 'expected', 424
		continue
	    
		

	fields=line.split(',')

	cg_timestamp = None
	cg_station   = None
	if uscg:
	    cg_timestamp = int(fields[-1]) # US Coast Guard time stamp.
	    #print len(fields),fields
	    for i in range(len(fields)-1,5,-1):
		if 0<len(fields[i]) and 'r' == fields[i][0]:
		    cg_station = fields[i]
		    break # Found it so ditch the for loop

	#print station
	#sys.exit('stations please work')

	ins = None

	try:
	    if   msgNum==1: ins = ais.ais_msg_1.sqlInsert(ais.ais_msg_1.decode(bv),dbType='sqlite')
	    elif msgNum==2: ins = ais.ais_msg_2.sqlInsert(ais.ais_msg_2.decode(bv),dbType='sqlite')
	    elif msgNum==3: ins = ais.ais_msg_3.sqlInsert(ais.ais_msg_3.decode(bv),dbType='sqlite')
	    elif msgNum==5: ins = ais.ais_msg_5.sqlInsert(ais.ais_msg_5.decode(bv),dbType='sqlite')
            else:
		print 'Warning... not handling type',msgNum,'line:',lineNum
		continue
	except:
	    print 'ERROR:  some decode error?','line:',lineNum
	    print '  ',line
	    continue

	counts[msgNum] += 1

	if uscg:
	    if None != cg_timestamp: ins.add('cg_timestamp',cg_timestamp)
	    if None != cg_station:   ins.add('cg_r',        cg_station)
	if verbose: print str(ins)
	try:
	    cu.execute(str(ins))
	except :
	    print 'ERROR: sql error?','line:',lineNum
	    print '  ', str(ins)
	    print '  ',line
    print counts
    cx.commit()
	

############################################################
if __name__=='__main__':
	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options] file1.ais [file2.ais ...]",version="%prog "+__version__)

	parser.add_option('-d','--database-file',dest='databaseFilename',default='ais.db3',
			  help='Name of the python file to write [default: %default]')
	parser.add_option('-C','--with-create',dest='createTables',default=False, action='store_true',
			  help='Do not create the tables in the database')

	parser.add_option('--without-uscg',dest='uscgTail',default=True,action='store_false',
			  help='Do not look for timestamp and receive station at the end of each line [default: with-uscg]')

	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	(options,args) = parser.parse_args()
	cx = sqlite.connect(options.databaseFilename)

	if options.createTables:
	    createTables(cx,verbose=options.verbose)

	if len(args)==0:
	    print 'processing from stdin'
    
	    loadData(cx,sys.stdin,verbose=options.verbose,uscg=options.uscgTail)
	else:
	    for filename in args:
		print 'processing file:',filename
		loadData(cx,file(filename,'r'),verbose=options.verbose,uscg=options.uscgTail)
